import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, ElementRef, Component, ChangeDetectionStrategy, Input, Output, ViewChild, HostBinding, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { timer } from 'rxjs';
import codemirror from 'codemirror';
import { CommonModule } from '@angular/common';

class CodeMirrorComponent {
    constructor(elementRef, ngZone, renderer, _differs) {
        this.elementRef = elementRef;
        this.ngZone = ngZone;
        this.renderer = renderer;
        this._differs = _differs;
        this.value = null;
        this.onTouchedCallback = () => { };
        this.onChangeCallback = () => { };
        this.autoMaxHeight = 0;
        this.delayRefreshTime = 200;
        this.focusChange = new EventEmitter();
        this.codemirrorClassName = true;
    }
    get codeMirrorGlobal() {
        if (this._codeMirror) {
            return this._codeMirror;
        }
        this._codeMirror = typeof CodeMirror !== 'undefined' ? CodeMirror : codemirror;
        return this._codeMirror;
    }
    ngOnInit() {
        if (!this._differ && this.options) {
            this._differ = this._differs.find(this.options).create();
        }
        this.initializeCodemirror();
        timer(this.delayRefreshTime).subscribe(() => {
            if (this.autoMaxHeight) {
                this.initializeAutoMaxHeight();
            }
            this.editor.refresh();
        });
    }
    ngOnChanges(simpleChanges) {
        const optionsChange = simpleChanges.options;
        if (optionsChange && !optionsChange.firstChange) {
            const changes = this._differ.diff(this.options);
            if (changes) {
                changes.forEachChangedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachAddedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachRemovedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
            }
        }
    }
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    writeValue(value) {
        const valid = value !== null && value !== undefined;
        if (this.editor && valid && this.value !== value) {
            this.editor.setValue(value);
            this.editor.clearHistory();
        }
        this.value = value;
    }
    initializeCodemirror() {
        this.ngZone.runOutsideAngular(() => {
            if (!this.options) {
                throw new Error("options is required");
            }
            this.editor = this.codeMirrorGlobal.fromTextArea(this.textAreaRef.nativeElement, this.options);
            this.editor.on("focus", () => this.ngZone.run(() => this.focusChange.emit(true)));
            this.editor.on("blur", () => {
                this.ngZone.run(() => this.focusChange.emit(false));
                this.editor.setCursor(0, null, { scroll: false });
            });
            this.editor.on("change", (cm, change) => {
                if (change.origin !== "setValue") {
                    this.value = cm.getValue();
                    this.onChangeCallback(this.value);
                }
            });
        });
    }
    initializeAutoMaxHeight() {
        this._codeWrapElement = this.elementRef.nativeElement.querySelector(".CodeMirror-code");
        this._codemirrorContentObserver = new MutationObserver(() => {
            this.applyEditorHeight();
        });
        this._codemirrorContentObserver.observe(this._codeWrapElement, {
            childList: true,
        });
    }
    applyEditorHeight() {
        if (this._codeWrapElement.offsetHeight >= this.autoMaxHeight) {
            const wrapHeight = this.elementRef.nativeElement.querySelector(".CodeMirror");
            if (wrapHeight.offsetHeight > this.autoMaxHeight) {
                this.editor.setSize("100%", `${this.autoMaxHeight}px`);
            }
        }
        else {
            this.editor.setSize("100%", `auto`);
        }
    }
    setOptionIfChanged(optionName, newValue) {
        if (!this.editor) {
            return;
        }
        // cast to any to handle strictly typed option names
        // could possibly import settings strings available in the future
        this.editor.setOption(optionName, newValue);
    }
    ngOnDestroy() {
        if (this._codemirrorContentObserver) {
            this._codemirrorContentObserver.disconnect();
        }
    }
}
CodeMirrorComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodeMirrorComponent, deps: [{ token: i0.ElementRef }, { token: i0.NgZone }, { token: i0.Renderer2 }, { token: i0.KeyValueDiffers }], target: i0.ɵɵFactoryTarget.Component });
CodeMirrorComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.3.8", type: CodeMirrorComponent, selector: "ng-codemirror, [ngCodeMirror]", inputs: { autoMaxHeight: "autoMaxHeight", options: "options", delayRefreshTime: "delayRefreshTime" }, outputs: { focusChange: "focusChange" }, host: { properties: { "class.ng-codemirror": "this.codemirrorClassName" } }, providers: [{
            provide: NG_VALUE_ACCESSOR,
            useExisting: forwardRef(() => CodeMirrorComponent),
            multi: true
        }], viewQueries: [{ propertyName: "textAreaRef", first: true, predicate: ["textAreaRef"], descendants: true, read: ElementRef, static: true }], usesOnChanges: true, ngImport: i0, template: `<textarea #textAreaRef style="display:none;"></textarea>`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodeMirrorComponent, decorators: [{
            type: Component,
            args: [{
                    selector: "ng-codemirror, [ngCodeMirror]",
                    template: `<textarea #textAreaRef style="display:none;"></textarea>`,
                    preserveWhitespaces: false,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => CodeMirrorComponent),
                            multi: true
                        }]
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i0.NgZone }, { type: i0.Renderer2 }, { type: i0.KeyValueDiffers }]; }, propDecorators: { autoMaxHeight: [{
                type: Input
            }], options: [{
                type: Input
            }], delayRefreshTime: [{
                type: Input
            }], focusChange: [{
                type: Output
            }], textAreaRef: [{
                type: ViewChild,
                args: ["textAreaRef", { read: ElementRef, static: true }]
            }], codemirrorClassName: [{
                type: HostBinding,
                args: ["class.ng-codemirror"]
            }] } });

class CodemirrorModule {
}
CodemirrorModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodemirrorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
CodemirrorModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodemirrorModule, declarations: [CodeMirrorComponent], imports: [CommonModule], exports: [CodeMirrorComponent] });
CodemirrorModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodemirrorModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.3.8", ngImport: i0, type: CodemirrorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [CodeMirrorComponent],
                    exports: [CodeMirrorComponent]
                }]
        }] });

/*
 * Public API Surface of ng-codemirror
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodeMirrorComponent, CodemirrorModule };
//# sourceMappingURL=ng-codemirror.mjs.map
